#include <iostream>
#include <GL/glut.h>

#include "CMove.h"
#include "Pieces.h"
#include "CBoard.h"




/////////////
//         //
//  EMPTY  //
//         //
/////////////

char CEmpty::GetChar() const
{
  if ((GetX() + GetY()) % 2 == 0)
    return '*';
  else
    return ' ';
}

type_piece CEmpty::GetType() const
{
  return EMPTY_T;
}

CBasePiece* CEmpty::GetCopy(CBoard* par) const
{
  return new CEmpty(par, GetX(), GetY());
}


bool CEmpty::IsCheck(type_color Col) const
{
  return false;
}

bool CEmpty::IsMoveValid(const CMove& move) const
{
  return false;
}

void CEmpty::MoveAlert(const CMove& move)
{
  return;
}
