#ifndef CEXCEPTION
#define CEXCEPTION

class CFatalError
{
  std::string ErrorString;

 public:
  CFatalError(const std::string Mess) : ErrorString(Mess) {}
  std::string GetErrorString() const { return ErrorString; }
};

#define MEMORY_ERROR CFatalError(std::string("Not sufficient memory.\n"))
#define WINDOW_ERROR CFatalError(std::string("Unable to create window or subwindow.\n"));

#endif
