#ifndef CSTARTUPWIN_H
#define CSTARTUPWIN_H

class CStartUpWin : public CWindow
{
 public:
  CStartUpWin(const std::string& caption = std::string("Welcome To ChessMart"),
              int width = -1, int height = -1);
  virtual void init(void);

  void Display(void);
  void Keyboard(unsigned char key, int x, int y);
  void Reshape(int width, int height);
};

extern CStartUpWin* p_StartUpWin;

#endif
