#ifndef CMAINWIN_H
#define CMAINWIN_H

class CMainWin : public CWindow
{
  CSectionBoard* BoardWin;
  std::string ChessTexture;
  std::string BoardTexture;

 public:
  CBoard& GetBoard() { return BoardWin->GetBoard(); }

  CMainWin(const std::string& caption = std::string(""),
           int width = -1, int height = -1);
  virtual ~CMainWin() { if (BoardWin) delete BoardWin; }
  virtual void init(void);

  void Display(void);
  void Keyboard(unsigned char key, int x, int y);
  void Reshape(int width, int height);
  void Mouse(int button, int state, int x, int y);
  void Motion(int x, int y);
};

extern CMainWin* p_MainWin;

#endif
